/**
  *  \file flak/flakmod.cc
  *  \brief FLAK Configuration Module Initializer
  *
  *  This is separate from flakconf.cc, so that flakconf.o has no
  *  dependency on spec_file_dir etc. This is required to be able to
  *  build the PDK-based FLAK server.
  *
  *  Unfortunately, this split also defeats the automatic analysis of
  *  scripts/checkinits.pl, because referencing the FLAK configuration
  *  does not automatically pull in the module initializer. So, be
  *  careful!
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "flak/flakconf.h"
#include "util/modinit.h"
#include "game/gamedirs.h"


void
loadFlakConfig()
{
    initFlakConfig();

    Ptr<Stream> f = spec_file_dir->openFileNT("pconfig.src", Stream::C_READ);
    bool sec = false;
    if (!f) {
        f = spec_file_dir->openFileNT("flak.src", Stream::C_READ);
        sec = true;
    }
    if (f) {
        loadFlakConfig(*f, sec);
    }
}

// There is no hard technical reason for this being read after mod_config, but
// it may simplify things (and error messages).
// Constraints: after mod_config
extern const ModuleInitializer mod_flak[] = {
    // Load default config upon startup and when leaving a directory
    { OnStartup | OnEnterNoGame, initFlakConfig },
    // Load config on entering directory
    { OnEnterGame, loadFlakConfig },
    { 0, 0 }
};
