/**
  *  \file int/if/shipif.h
  *  \brief Interpreter Game Interface: Ships
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_SHIPIF_H
#define PCC_V2_INT_IF_SHIPIF_H

#include "int/compound.h"
#include "int/context.h"

class GShip;

IntValue*   IFShipGet(IntArgBlock& args);
IntContext* IFShipMake();

/** Ship context. */
class IntShipContext : public IntContext {
    int sid;
 public:
    IntShipContext(int sid);
    ~IntShipContext();

    // IntContext:
    int32_t   lookup(string_t name);
    void      set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    bool      next();
    IntContext* clone();
    GObject*  getObject();
    string_t  toString(bool readable);
    void      store(IntTagNode& sv, Stream& aux);
};

/** Definition of ship properties. */
enum IntShipProperty {
    ispAuxId,
    ispAuxAmmo,
    ispAuxCount,
    ispAuxShort,
    ispAuxName,
    ispBeamId,
    ispBeamCount,
    ispBeamShort,
    ispBeamName,
    ispCargoColonists,
    ispCargoD,
    ispCargoFree,
    ispCargoM,
    ispCargoMoney,
    ispCargoN,
    ispCargoStr,
    ispCargoSupplies,
    ispCargoT,
    ispCrew,
    ispDamage,
    ispEnemyId,
    ispEngineId,
    ispEngineName,
    ispFCode,
    ispFighterBays,
    ispFighterCount,
    ispFleetId,
    ispFleetName,
    ispFleetStatus,
    ispFleet,
    ispHeadingAngle,
    ispHeadingName,
    ispHullSpecial,
    ispId,
    ispLevel,
    ispLocX,
    ispLocY,
    ispLoc,
    ispMarked,
    ispMass,
    ispMissionId,
    ispMissionIntercept,
    ispMissionShort,
    ispMissionTow,
    ispMissionName,
    ispMoveETA,
    ispMoveFuel,
    ispName,
    ispOrbitId,
    ispOrbitName,
    ispPlayed,
    ispRealOwner,
    ispSpeedId,
    ispSpeedName,
    ispTask,
    ispTorpId,
    ispTorpCount,
    ispTorpLCount,
    ispTorpShort,
    ispTorpName,
    ispTransferShipColonists,
    ispTransferShipD,
    ispTransferShipId,
    ispTransferShipM,
    ispTransferShipN,
    ispTransferShipName,
    ispTransferShipSupplies,
    ispTransferShipT,
    ispTransferShip,
    ispTransferUnloadColonists,
    ispTransferUnloadD,
    ispTransferUnloadId,
    ispTransferUnloadM,
    ispTransferUnloadN,
    ispTransferUnloadName,
    ispTransferUnloadSupplies,
    ispTransferUnloadT,
    ispTransferUnload,
    ispTypeChar,
    ispTypeStr,
    ispWaypointDistance,
    ispWaypointDX,
    ispWaypointDY,
    ispWaypointPlanetId,
    ispWaypointX,
    ispWaypointY,
    ispWaypointName
};

IntValue* getShipProperty(const GShip& sh, IntShipProperty isp);
void      setShipProperty(GShip& sh, IntShipProperty isp, IntValue* value);

#endif
