%
%  Assembler test
%
%  This tests assembling all instructions. To check it, use teststmt,
%  load it using '.load', and manually check disassembly of all subs.
%

% All unary operations in increasing order of minor op
sub unary
  unot
  ubool
  uneg
  upos
  usin
  ucos
  utan
  uzap
  uabs
  uexp
  ulog
  ubitnot
  uisempty
  uisnum
  uisstr
  uasc
  uchr
  ustr
  usqrt
  utrunc
  uround
  ultrim
  urtrim
  ulrtrim
  ulength
  uval
  utrace
  unot2
  uatom
  uatomstr
  ukeycreate
  ukeylookup
  uinc
  udec
  uisproc
endsub

% All binary operations in increasing order of minor op
sub binary
  band
  bor
  bxor
  badd
  bsub
  bmul
  bdiv
  bidiv
  brem
  bpow
  bconcat
  bconcempty
  bcmpeq
  bcmpeqNC
  bcmpne
  bcmpneNC
  bcmple
  bcmpleNC
  bcmplt
  bcmpltNC
  bcmpge
  bcmpgeNC
  bcmpgt
  bcmpgtNC
  bmin
  bminNC
  bmax
  bmaxNC
  bfirststr
  bfirststrNC
  breststr
  breststrNC
  bfindstr
  bfindstrNC
  bbitand
  bbitor
  bbitxor
  bstr
  batan
  blcut
  brcut
  bendcut
  bstrmult
  bkeyaddparent
  bkeyfind
endsub

% All ternary operations in increasing order of minor op
sub ternary
  tkeyadd
endsub

% All valid push operations
sub push
  .local NamedLocal
  pushvar NamedVar
  pushloc NamedLocal
  pushtop #42
  pushglob #17
  pushgvar NamedGlobal
  pushlit 'foo'
  pushlit 99
  pushlit -42
  pushlit true
  pushint 42
  pushint -99
  pushbool 0
  pushbool 1
  pushbool -1
  pushsub ternary
endsub

% All valid store operations (note that the assembler will accept more)
sub store
  .local NamedLocal
  storevar NamedVar
  storeloc NamedLocal
  storetop #42
  storeglob #17
  storegvar NamedGlobal
endsub

% All valid pop operations (note that the assembler will accept more)
sub pop
  .local NamedLocal
  popvar NamedVar
  poploc NamedLocal
  poptop #42
  popglob #17
  popgvar NamedGlobal
endsub

% All valid jump instructions with absolute arguments
.jumps abs
sub absoluteJump
  j foo
  je foo
  jt foo
  jte foo
  jf foo
  jfe foo
  jtf foo
  label foo

  jp foo
  jep foo
  jtp foo
  jtep foo
  jfp foo
  jfep foo
  jtfp foo
  jneverp foo
foo:
  nop
  j foo
  catch foo
  jdz foo
endsub

% All valid jump instructions with symbolic arguments
.jumps sym
sub symbolicJump
  j foo
  je foo
  jt foo
  jte foo
  jf foo
  jfe foo
  jtf foo
  label foo

  jp foo
  jep foo
  jtp foo
  jtep foo
  jfp foo
  jfep foo
  jtfp foo
  jneverp foo
foo:
  nop
  j foo
  catch foo
  jdz foo
endsub

% All valid indirect operations
sub indirect
  callind 5
  loadind 5
  storeind 5
  popind 5
  procind 5
  ploadind 5
  pstoreind 5
  ppopind 5
  fcallind 5
  funcind 5
  fstoreind 5
  fpopind 5
endsub

% All valid member-reference operations
sub member
  callmem foo
  loadmem foo
  storemem foo
  popmem foo
endsub

% All valid 'dim' operations
sub dim
  dimloc local
  dimglob shared
  dimtop static
endsub

% All valid special instructions
sub special
  suncatch
  sreturn 1
  swith
  sendwith
  sfirstindex
  snextindex
  sendindex
  sevals 10
  sevalx
  sdefsub foo
  sdefshipp ShipProperty
  sdefplanetp PlanetProperty
  sload
  sprint
  saddhook
  saddhook 99
  srunhook
  srunhook 98
  sthrow
  sterminate
  ssuspend
endsub

% All valid stack operations
sub stack
  drop 10
  swap 10
  dup 10
endsub

% A zoo of pseudo-operations
sub pseudos
  % locals
  .local a
  .local b
  .local c

  % literals, re-used and new
  .lit 1
  .lit 2
  .lit 3
  .lit 2
  .lit !2
  .lit new 1
  .lit "string"
  .lit (130,5)

  % symbols
  .sym one
  .sym two

  % sub references
  .sub stack

  % rename this routine
  .name pseudoOpTest

  .defsubs

  % reference all locals and literals
  pushloc #0 % a
  pushloc #1 % b
  pushloc #2 % c

  pushlit #0 % 1
  pushlit #1 % 2
  pushlit #2 % 3
  pushlit #3 % 2
  pushlit #4 % 1
  pushlit #5 % "string"
  pushlit #6 % Minefield(5)

  pushvar #0 % one
  pushvar #1 % two

  pushsub #0 % stack

  pushlit 1  % this recycles a literal
  pushlit new 1 % this makes a new one
endsub

% Generic instructions
sub generic
  .local a
  gen0.0
  gen1.1 #1
  genint2.2 1
  genloc3.3 a
  gensym4.4 b
  gensub5.5 stack
label:
  genlabel6.6 label
  genlit47.11 999
endsub

% All valid literals
sub literals
  % int
  pushlit 0
  pushlit -1
  pushlit -999999999
  pushlit +999999999
  pushlit 999999999

  % float
  pushlit 1.5
  pushlit 1.0
  pushlit 10000000000000.
  pushlit .001
  pushlit -.001
  pushlit +.001
  pushlit 0.001

  % string
  pushlit ''
  pushlit ""
  pushlit 'foobar'
  pushlit 'foo\bar'
  pushlit "foobar"
  pushlit "foo\\bar"

  % bools and null
  pushlit true
  pushlit false
  pushlit null

  % hacks
  pushlit (130,5)
endsub

sub main
  .defsubs
endsub
